@@ -67,7 +67,7 @@ let check_forbidden_chars (uc : Charset.uchar) =
 
 let canonize_basename name =
   let buf = Buffer.create 100 in
-  let uname = Charset.to_utf8 name in
+  let uname = Charset.Locale.to_utf8 name in
   for i = 0 to Charset.utf8_length uname - 1 do
     (* replace chars on users request *)
     let uc = Charset.utf8_get uname i in
@@ -85,7 +85,7 @@ let canonize_basename name =
       end
   done;
   if not Autoconf.windows then
-    Charset.to_locale (Buffer.contents buf)
+    Charset.Locale.to_locale (Buffer.contents buf)
   else
     Buffer.contents buf (* Windows uses patched Ocaml which always uses Unicode chars *)
 
@@ -107,27 +107,33 @@ let send_dirfull_warning dir full line1 =
     let send_mail_again =
       try
         let last = Hashtbl.find last_sent_dir_warning dir in
-	last < time_threshold
-      with Not_found -> true in
-
+        last < time_threshold
+      with Not_found -> true
+    in
     if send_mail_again then begin
       if full then Hashtbl.replace last_sent_dir_warning dir current_time;
       CommonEvent.add_event (Console_message_event
         (Printf.sprintf "\nWARNING: %s %s, %s\n" dir status line1));
-      if !!mail <> "" then
+      match String2.tokens !!mail with
+      | [] -> ()
+      | mails ->
         let module M = Mailer in
         let subject = Printf.sprintf "[mldonkey@%s] AUTOMATED WARNING: %s %s" (Unix.gethostname ()) dir status in
         let mail = {
-          M.mail_to = !!mail; M.mail_from = !!mail;
-	  M.mail_subject = subject; M.mail_body = line1;
+          M.mail_to = mails;
+          M.mail_from = List.hd mails;
+          M.mail_subject = subject;
+          M.mail_body = line1;
+          M.smtp_login = !!smtp_login;
+          M.smtp_password = !!smtp_password;
         } in
-	try
+        try
           M.sendmail !!smtp_server !!smtp_port !!add_mail_brackets mail
-	with _ -> ()
+        with _ -> ()
     end
   end
 
-let file_commited_name incoming_dir file =
+let file_committed_name incoming_dir file =
   (try Unix2.safe_mkdir incoming_dir with _ -> ());
   let fs = Unix32.filesystem incoming_dir in
   let namemax =
@@ -151,7 +157,7 @@ let file_commited_name incoming_dir file =
       in
       iter 1
     else new_name in
-  set_file_best_name file (file_best_name file) "" 0;
+  set_file_best_name file (file_best_name file) fs namemax;
   Filename.concat incoming_dir new_name
 
 let script_for_file file incoming new_name =
@@ -203,7 +209,7 @@ let script_for_file file incoming new_name =
 	    ("USER_MAIL", ( if (file_owner file).user_mail <> "" then
                               (file_owner file).user_mail
                             else
-                              if !!mail <> "" then !!mail else ""));
+                              match String2.tokens !!mail with [] -> "" | x::_ -> x));
 	    ("FILE_GROUP_CNT", string_of_int (fst (file_group_info)));
 	    ]
             @ snd (file_group_info))
@@ -240,7 +246,7 @@ let file_commit file =
             ()
           in
 
-	  let new_name = file_commited_name incoming.shdir_dirname file in
+	  let new_name = file_committed_name incoming.shdir_dirname file in
 	    if Unix2.is_directory file_name then begin
 	      Unix2.safe_mkdir new_name;
 	      Unix2.chmod new_name !Unix32.create_dir_mode;
@@ -268,7 +274,7 @@ let file_commit file =
                       incoming.shdir_dirname incoming.shdir_priority
                       best_name new_name);
               with e ->
-                  lprintf_nl "Exception %s while trying to share commited file"
+                  lprintf_nl "Exception %s while trying to share committed file"
                     (Printexc2.to_string e);
             end;
 
@@ -326,7 +332,8 @@ let file_cancel file user =
 
 let mail_for_completed_file file =
   let usermail = (file_owner file).user_mail in
-  if (!!mail <> "" || usermail <> "") && !!smtp_server <> "" && !!smtp_port <> 0 then begin
+  let mail = String2.tokens !!mail in
+  if (mail <> [] || usermail <> "") && !!smtp_server <> "" && !!smtp_port <> 0 then begin
     let module M = Mailer in
     let info = file_info file in
     let line1 = "mldonkey has completed the download of:\r\n\r\n" in
@@ -343,9 +350,9 @@ let mail_for_completed_file file =
     in
 
     let subject = if !!filename_in_subject then
-        Printf.sprintf "[mldonkey@%s] file received - %s" (Unix.gethostname ()) (file_best_name file)
+        Printf.sprintf "mldonkey - %s complete" (file_best_name file)
       else
-        Printf.sprintf "mldonkey@%s, file received" (Unix.gethostname ())
+        Printf.sprintf "mldonkey - download complete"
     in
 
 (* TODO: This information can be wrong *)
@@ -365,20 +372,26 @@ let mail_for_completed_file file =
     in
 
     let line6 =
-      Printf.sprintf "\r\nUser/Group: %s:%s\r\n" (file_owner file).user_name (user2_print_group (file_group file))
+      Printf.sprintf "\r\nUser/Group: %s:%s" (file_owner file).user_name (user2_print_group (file_group file))
     in
-      
+
+    let line7 =
+      Printf.sprintf "\r\nHost: %s\r\n" (Unix.gethostname ())
+    in
+
     let send_mail address admin =
       let mail = {
         M.mail_to = address;
-        M.mail_from = address;
+        M.mail_from = List.hd address;
         M.mail_subject = subject;
-        M.mail_body = line1 ^ line2 ^ line3 ^ line4 ^ line5 ^ (if admin then line6 else "");
+        M.mail_body = line1 ^ line2 ^ line3 ^ line4 ^ line5 ^ (if admin then line6 else "") ^ line7;
+        M.smtp_login = !!smtp_login;
+        M.smtp_password = !!smtp_password;
       } in
         M.sendmail !!smtp_server !!smtp_port !!add_mail_brackets mail
     in
-    if !!mail <> "" then send_mail !!mail true; (* Multiuser ToDo: this mail is for the admin user, optional? *)
-    if usermail <> "" && usermail <> !!mail then (try send_mail usermail false with Not_found -> ())
+    if mail <> [] then send_mail mail true; (* Multiuser ToDo: this mail is for the admin user, optional? *)
+    if usermail <> "" && [usermail] <> mail then (try send_mail [usermail] false with Not_found -> ())
   end
 
 let file_completed (file : file) =
@@ -499,6 +512,7 @@ let contact_remove c =
       lprintf_nl "Exception in contact_remove: %s" (Printexc2.to_string e)
 
 let clean_exit n =
+  begin
   let can_exit = networks_for_all network_clean_exit in
   if can_exit then exit_properly n
   else 
@@ -508,7 +522,15 @@ let clean_exit n =
         if can_exit || retry_counter > !!shutdown_timeout then
 	  exit_properly n
 	else retry_later (retry_counter + 1)) in
-    retry_later 0
+    retry_later 0;
+
+  if (upnp_port_forwarding ()) then
+    begin
+    if !!clear_upnp_port_at_exit then
+      UpnpClient.remove_all_maps 0 ;
+    UpnpClient.job_stop 3;
+    end;
+  end
 
 let time_of_sec sec =
   let hours = sec / 60 / 60 in
@@ -569,7 +591,7 @@ let start_download file =
 	    ("USER_MAIL", ( if (file_owner file).user_mail <> "" then
                               (file_owner file).user_mail
                             else
-                              if !!mail <> "" then !!mail else ""));
+                              match String2.tokens !!mail with [] -> "" | x::_ -> x));
 	    ("FILE_GROUP_CNT", string_of_int (fst (file_group_info)));
 	    ]
             @ snd (file_group_info))
