@@ -76,7 +76,7 @@ let real_startup_message () =
   let s =
   !startup_message ^ (verify_user_admin ()) ^ (check_supported_os ()) 
   ^ (if not !dns_works then "DNS resolution does not work\n" else "")
-  ^ (if not !Charset.conversion_enabled then "Charset conversion does not work, disabled\n" else "")
+  ^ (if not !Charset.Locale.conversion_enabled then "Charset conversion does not work, disabled\n" else "")
   ^ (match !created_new_base_directory with
        None -> ""
      | Some dir -> (Printf.sprintf "MLDonkey created a new home directory in %s\n" dir))
@@ -718,7 +718,7 @@ let ctd fn td = Printf.sprintf "\\<td onClick=\\\"location.href='submit?q=vd+%d'
           (if !!html_mods_use_js_tooltips then
                         Printf.sprintf "
                                 onMouseOver=\\\"mOvr(this);setTimeout('popLayer(\\\\\'%s<br>%sFile#: %d<br>Network: %s<br>User%s %s%s%s\\\\\')',%d);setTimeout('hideLayer()',%d);return true;\\\" onMouseOut=\\\"mOut(this);hideLayer();setTimeout('hideLayer()',%d)\\\"\\>"
-                        (Http_server.html_real_escaped (Charset.to_utf8 file.file_name))
+                        (Http_server.html_real_escaped (Charset.Locale.to_utf8 file.file_name))
 			(match file_magic (file_find file.file_num) with
 			   None -> ""
 			 | Some magic -> "File type: " ^ (Http_server.html_real_escaped magic) ^ "<br>")
@@ -739,7 +739,7 @@ let ctd fn td = Printf.sprintf "\\<td onClick=\\\"location.href='submit?q=vd+%d'
 	     else
 	       file.file_comments
 	   in
-           List.iter (fun (_,_,_,s) -> Printf.bprintf buf1 "%s<br>" (Http_server.html_real_escaped (Charset.to_utf8 s))) comments;
+           List.iter (fun (_,_,_,s) -> Printf.bprintf buf1 "%s<br>" (Http_server.html_real_escaped (Charset.Locale.to_utf8 s))) comments;
 			      Buffer.contents buf1
 			    end)
 
@@ -1170,10 +1170,10 @@ let display_file_list buf o l =
         if !!auto_commit then
           if o.conn_output = HTML then
             html_mods_table_one_row buf "searchTable" "search" [
-              ("", "srh", "Files will be automatically commited in the incoming directory"); ]
+              ("", "srh", "Files will be automatically committed in the incoming directory"); ]
           else
             Printf.bprintf buf
-              "Files will be automatically commited in the incoming directory"
+              "Files will be automatically committed in the incoming directory"
         else
           if o.conn_output = HTML then
             html_mods_table_one_row buf "searchTable" "search" [
@@ -1295,10 +1295,10 @@ let old_print_search buf o results =
                 match r.result_names with
                   [] -> ()
                 | name :: names ->
-                    Printf.bprintf buf "%s\n" (shorten name !!max_name_len);
+                    Printf.bprintf buf "%s\n" (shorten name !!max_result_name_len);
                     List.iter (fun s ->
                         if use_html_mods o then Printf.bprintf buf "\\<BR\\>";
-                        Printf.bprintf buf "       %s\n" (shorten s !!max_name_len)
+                        Printf.bprintf buf "       %s\n" (shorten s !!max_result_name_len)
                     ) names;
               end;
               if r.result_done then
@@ -1630,7 +1630,7 @@ let print_results stime buf o results =
                           if o.conn_output = HTML then Buffer.add_string buf "\\</table\\>\n";
 
                           Buffer.contents buf
-                    ) !!max_name_len)
+                    ) !!max_result_name_len)
                   (if o.conn_output = HTML then
                       begin
                         if !!html_mods then "\\</a\\>\\</td\\>"
@@ -1671,13 +1671,13 @@ let print_results stime buf o results =
     begin
 
       html_mods_table_header buf "resultsTable" "results" [
-		( "1", "srh", "Number", "#" ) ;
-		( "1", "srh ar", "Size", "Size" ) ;
-		( "0", "srh ar", "Availability", "A" )  ;
-		( "0", "srh ar", "Status, D = already downloaded", "S" )  ;
-		( "0", "srh", "Filename", "Name" ) ;
-		( "0", "srh", "Tag", "Tag" ) ;
-		( "0", "srh", "MD4", "MD4" ) ];
+		( Num, "srh", "Number", "#" ) ;
+		( Num, "srh ar", "Size", "Size" ) ;
+		( Str, "srh ar", "Availability", "A" )  ;
+		( Str, "srh ar", "Status, D = already downloaded", "S" )  ;
+		( Str, "srh", "Filename", "Name" ) ;
+		( Str, "srh", "Tag", "Tag" ) ;
+		( Str, "srh", "MD4", "MD4" ) ];
 
       print_table_html_mods buf
        (List.rev !files)
@@ -1733,16 +1733,16 @@ let browse_friends () =
 
 let networks_header buf =
     html_mods_table_header buf "networkTable" "networkInfo" [
-      ( "0", "srh br", "Network name", "Network" ) ;
-      ( "0", "srh br", "Status", "Status" ) ;
-      ( "0", "srh br", "Has upload", "Upload" ) ;
-      ( "0", "srh br", "Has servers", "Servers" ) ;
-      ( "0", "srh br", "Has supernodes", "Supernodes" ) ;
-      ( "0", "srh br", "Has search", "Search" ) ;
-      ( "0", "srh br", "Has chat", "Chat" ) ;
-      ( "0", "srh br", "Has rooms", "Rooms" ) ;
-      ( "0", "srh", "Has multinet", "Multinet" ) ;
-      ( "0", "srh", "Has porttest", "Porttest" ) ]
+      ( Str, "srh br", "Network name", "Network" ) ;
+      ( Str, "srh br", "Status", "Status" ) ;
+      ( Str, "srh br", "Has upload", "Upload" ) ;
+      ( Str, "srh br", "Has servers", "Servers" ) ;
+      ( Str, "srh br", "Has supernodes", "Supernodes" ) ;
+      ( Str, "srh br", "Has search", "Search" ) ;
+      ( Str, "srh br", "Has chat", "Chat" ) ;
+      ( Str, "srh br", "Has rooms", "Rooms" ) ;
+      ( Str, "srh", "Has multinet", "Multinet" ) ;
+      ( Str, "srh", "Has porttest", "Porttest" ) ]
 
 let print_network_modules buf o =
   let buf = o.conn_buf in
@@ -1932,7 +1932,7 @@ let buildinfo html buf =
 	    " gd(jpg)"
 	| _, false, false ->
 	    " gd(neither jpg nor png ?)") ^
-      (match Autoconf.has_iconv, !Charset.conversion_enabled with
+      (match Autoconf.has_iconv, !Charset.Locale.conversion_enabled with
        | true, true -> " iconv(active)"
        | true, false -> " iconv(inactive)"
        | false, _ -> " no-iconv") ^
@@ -1940,14 +1940,15 @@ let buildinfo html buf =
        | true, true -> " magic(active)"
        | true, false -> " magic(inactive)"
        | false, _ -> " no-magic") ^
+      (if Autoconf.upnp_natpmp then " upnp natpmp" else " no-upnp no-natpmp") ^
       (if Autoconf.check_bounds then " check-bounds" else " no-check-bounds")
     );
   let list = List.rev !list in
      
     if html then
       html_mods_table_header buf "sharesTable" "shares" [
-       ( "0", "srh", "core Build informations", "Buildinfo" ) ;
-       ( "0", "srh", "", "" ) ]
+       ( Str, "srh", "core Build information", "Buildinfo" ) ;
+       ( Str, "srh", "", "" ) ]
     else
       Printf.bprintf buf "\n\t--Buildinfo--\n";
     html_mods_cntr_init ();
@@ -2047,8 +2048,8 @@ let runinfo html buf o =
     ( 
       "",
       Printf.sprintf "\t\t language: %s - locale: %s - UTC offset: %s"
-	Charset.default_language
-	Charset.locstr
+	Charset.Locale.default_language
+	Charset.Locale.locale_string
 	(Rss_date.mk_timezone (Unix.time ()))
     );
   tack list
@@ -2075,8 +2076,8 @@ let runinfo html buf o =
 
     if html then
       html_mods_table_header buf "sharesTable" "shares" [
-       ( "0", "srh", "core runtime informations", "Runinfo" ) ;
-       ( "0", "srh", "", "" ) ]
+       ( Str, "srh", "core runtime information", "Runinfo" ) ;
+       ( Str, "srh", "", "" ) ]
     else
       Printf.bprintf buf "\n\t--Runinfo--\n";
     html_mods_cntr_init ();
@@ -2112,9 +2113,9 @@ let portinfo html buf =
   let fill_network s = String.make (max 0 (!network_name_list_width - 7)) s in
   if html then
       html_mods_table_header buf "sharesTable" "shares" [
-       ( "0", "srh", "Network", "Network" ) ;
-       ( "0", "srh ar", "Port", "Port" ) ;
-       ( "1", "srh", "Type", "Type" ) ]
+       ( Str, "srh", "Network", "Network" ) ;
+       ( Num, "srh ar", "Port", "Port" ) ;
+       ( Str, "srh", "Type", "Type" ) ]
   else
     begin
       Printf.bprintf buf "\n\t--Portinfo--\n";
@@ -2158,12 +2159,12 @@ let diskinfo html buf =
   html_mods_cntr_init ();
   if html then
       html_mods_table_header buf "sharesTable" "shares" [
-       ( "0", "srh", "Directory", "Directory" ) ;
-       ( "0", "srh", "Directory type", "Type" ) ;
-       ( "1", "srh ar", "HDD used", "used" ) ;
-       ( "1", "srh ar", "HDD free", "free" ) ;
-       ( "1", "srh ar", "% free", "% free" ) ;
-       ( "0", "srh", "Filesystem", "FS" ) ]
+       ( Str, "srh", "Directory", "Directory" ) ;
+       ( Str, "srh", "Directory type", "Type" ) ;
+       ( Num, "srh ar", "HDD used", "used" ) ;
+       ( Num, "srh ar", "HDD free", "free" ) ;
+       ( Num, "srh ar", "% free", "% free" ) ;
+       ( Str, "srh", "Filesystem", "FS" ) ]
   else
     begin
       Printf.bprintf buf "\n\t--Diskinfo--\n";
@@ -2459,14 +2460,14 @@ let print_upstats o list server =
 
   if use_html_mods o then
     html_mods_table_header buf "upstatsTable" "upstats" [
-      ( "1", "srh", "Total file requests", "Reqs" ) ;
-      ( "1", "srh", "Total bytes sent", "Total" ) ;
-      ( "1", "srh", "Upload Ratio", "UPRatio" ) ;
-      ( "0", "srh", "Preview", "P" ) ;
-      ( "0", "srh", "Filename", "Filename" );
-      ( "0", "srh", "Statistic links", "Stats" );
-      ( "0", "srh", "Published on servers", "Publ" );
-      ( "0", "srh", "Share status", "Status" )
+      ( Num, "srh", "Total file requests", "Reqs" ) ;
+      ( Num, "srh", "Total bytes sent", "Total" ) ;
+      ( Num, "srh", "Upload Ratio", "UPRatio" ) ;
+      ( Str, "srh", "Preview", "P" ) ;
+      ( Str, "srh", "Filename", "Filename" );
+      ( Str, "srh", "Statistic links", "Stats" );
+      ( Str, "srh", "Published on servers", "Publ" );
+      ( Str, "srh", "Share status", "Status" )
     ]
   else
     begin
@@ -2492,7 +2493,7 @@ let print_upstats o list server =
 	Printf.bprintf buf "\\<tr class=\\\"dl-%d\\\"" (html_mods_cntr ());
 	(if !!html_mods_use_js_tooltips then
 	   Printf.bprintf buf " onMouseOver=\\\"mOvr(this);setTimeout('popLayer(\\\\\'%s<br>%s%s\\\\\')',%d);setTimeout('hideLayer()',%d);return true;\\\" onMouseOut=\\\"mOut(this);hideLayer();setTimeout('hideLayer()',%d)\\\"\\>"
-	    (Http_server.html_real_escaped (Filename.basename (Charset.to_utf8 impl.impl_shared_codedname)))
+	    (Http_server.html_real_escaped (Filename.basename (Charset.Locale.to_utf8 impl.impl_shared_codedname)))
 	    (match impl.impl_shared_file with
 	       None -> "no file info"
 	     | Some file -> match file_magic file with | None -> "no magic"
