@@ -61,7 +61,8 @@ let opt_type_to_pref_type o =
       | "Filename" -> BFilename
       | "Path" -> BPath
       | "Password" -> BPassword
-      | "Time" -> BTime
+      | "GraphTime" -> BCombo
+      | "Time" -> BInt
       | "Color" -> BColor
       | "Font" -> BFont
       | "Int" -> BInt
@@ -96,7 +97,6 @@ let string_to_section s =
     | "HTML mods" -> Some Interfaces
     | "MLgui" -> Some Interfaces
     | "MLguiDebug" -> Some Interfaces
-    | "MLChat" -> Some Mlchat
     | "Other" -> Some Other
     | "Paths" -> Some Tools
     | "Startup" -> Some Tools
@@ -130,7 +130,6 @@ let string_to_subsection s =
     | "HTML mods" -> Some Html_mods
     | "MLgui" -> Some Mlgui
     | "MLguiDebug" -> Some Debug
-    | "MLChat" -> None
     | "Other" -> None
     | "Paths" -> Some Paths
     | "Startup" -> Some Startup
@@ -168,23 +167,22 @@ let gift_reg = Str.regexp_case_fold ".*gift.*"
 let peer_reg = Str.regexp_case_fold ".*peer.*"
 let server_reg = Str.regexp_case_fold ".*server.*"
 let mail_reg = Str.regexp_case_fold ".*smtp.*\\|mail"
-let mlchat_server_reg = Str.regexp_case_fold ".*app.*"
 let tracker_reg = Str.regexp_case_fold ".*tracker.*"
 let overnet_reg = Str.regexp_case_fold ".*overnet.*"
 let shared_reg = Str.regexp_case_fold ".*shared.*"
 let display_reg = Str.regexp_case_fold ".*vd.*\\|.*show.*\\|.*availability.*"
 let look_reg = Str.regexp_case_fold ".*theme.*\\|.*style.*\\|.*checkbox.*\\|.*human_readable.*\\|html_mods"
-let mlchat_client_reg = Str.regexp_case_fold ".*addr.*\\|.*port.*"
 let dc_client_reg = Str.regexp_case_fold ".*login.*\\|.*firewalled.*\\|.*client.*"
 let dc_server_reg = Str.regexp_case_fold ".*hub.*\\|.*server*."
 let ed2k_client_reg = Str.regexp_case_fold ".*md4.*\\|.*login.*\\|.*high_id.*\\|.*port.*\\|.*max_xs.*\\|.*max_udp.*"
 let ed2k_upload_reg = Str.regexp_case_fold ".*upload.*\\|.*warning.*\\|.*queued.*"
 let ed2k_download_reg = Str.regexp_case_fold ".*download.*\\|.*sources_per_chunk.*"
-let ed2k_sources_reg = Str.regexp_case_fold ".*source.*\\|.*initial_score.*\\|.*ban.*\\|.*clients.*\\|.*client_connections_delay.*\\|.*client_rank.*\\|.*black_list.*\\|.*max_indirect.*"
+let ed2k_sources_reg = Str.regexp_case_fold ".*source.*\\|.*ban.*\\|.*clients.*\\|.*client_connections_delay.*\\|.*client_rank.*\\|.*black_list.*\\|.*max_indirect.*"
 let ed2k_server_reg = Str.regexp_case_fold ".*files_queries.*\\|.*connections_delay.*\\|.*protocol_version.*\\|.*server.*"
 let nap_server_reg = Str.regexp_case_fold ".*napigator.*\\|.*server.*"
 let slsk_client_reg = Str.regexp_case_fold ".*client_.*\\|.*login.*\\|.*password.*"
 let slsk_sources_reg = Str.regexp_case_fold ".*token.*\\|.*clients.*\\|.*cache_.*"
+let mlgui_connection_reg = Str.regexp_case_fold ".*gtk_connection.*"
 let mlgui_client_reg = Str.regexp_case_fold ".*gtk_client_.*"
 let mlgui_look_reg = Str.regexp_case_fold ".*gtk_look_.*"
 let mlgui_color_reg = Str.regexp_case_fold ".*gtk_color_.*"
@@ -207,16 +205,14 @@ let groups_reg =
         [(Some display_reg, Some Display_conf); (Some look_reg, Some Look);
          (None, Some General_)];
    (Some Interfaces, Some Mlgui),
-        [(Some mlgui_client_reg, Some Client); (Some mlgui_look_reg, Some Look);
-         (Some mlgui_color_reg, Some Colors); (Some mlgui_font_reg, Some Fonts);
-         (Some mlgui_graph_reg, Some Graph); (None, Some Others)];
+        [(Some mlgui_connection_reg, Some Proxy); (Some mlgui_client_reg, Some Client);
+         (Some mlgui_look_reg, Some Look); (Some mlgui_color_reg, Some Colors);
+         (Some mlgui_font_reg, Some Fonts);(Some mlgui_graph_reg, Some Graph);
+         (None, Some Others)];
    (Some Tools, Some Startup),
         [(Some gui_reg, Some Gui); (None, Some User)];
    (Some Tools, Some Mail),
         [(Some mail_reg, Some Mail_setup); (None, Some Others)];
-   (Some Mlchat, None),
-        [(Some mlchat_server_reg, Some Server); (Some mlchat_client_reg, Some Client);
-         (None, Some General_)];
    (Some Bittorrent, None),
         [(Some client_reg, Some Client); (Some tracker_reg, Some Tracker);
          (None, Some Others)];
@@ -321,6 +317,15 @@ match p.pref_subsection with
           | "gtk_client_lang" ->
               (p.pref_option_list <- U.languages;
                BCombo)
+          | "gtk_graph_time_downloads" ->
+              (p.pref_option_list <- ["quarter"; "hour"; "half_day"; "day"; "week"; "month"; "year"];
+               p.pref_type)
+          | "gtk_graph_time_uploads" ->
+              (p.pref_option_list <- ["quarter"; "hour"; "half_day"; "day"; "week"; "month"; "year"];
+               p.pref_type)
+          | "gtk_graph_time_file" ->
+              (p.pref_option_list <- ["quarter"; "hour"; "half_day"; "day"; "week"; "month"; "year"];
+               p.pref_type)
           | _ -> p.pref_type
       end
   | _ -> p.pref_type
@@ -381,6 +386,9 @@ let update_preferences list pv =
             let _ =
               match o.M.option_name with
                   "client_name" -> G.client_name := o.M.option_value
+                | "max_hard_download_rate" -> G.max_hard_download_rate := int_of_float (safe_int o.M.option_value)
+                | "max_hard_upload_rate" -> G.max_hard_upload_rate := int_of_float (safe_int o.M.option_value)
+                | "update_gui_delay" -> G.update_gui_delay := int_of_float (1000. *. safe_float o.M.option_value)
                 | _ -> ()
             in
             try
@@ -405,7 +413,6 @@ let section_to_label sec =
       Some Main -> !M.cW_lb_main
     | Some Interfaces -> !M.cW_lb_interfaces
     | Some Tools -> !M.cW_lb_tools
-    | Some Mlchat -> !M.cW_lb_mlchat
     | Some Other -> !M.cW_lb_other
     | Some Bittorrent -> !M.cW_lb_bittorrent
     | Some Direct_connect -> !M.cW_lb_direct_connect
@@ -486,7 +493,6 @@ let icon_from_section section =
   match section with
       Some Main -> A.get_icon ~icon:M.icon_type_source_normal ~size:A.LARGE ()
     | Some Interfaces -> A.get_icon ~icon:M.icon_menu_interfaces ~size:A.LARGE ()
-    | Some Mlchat -> A.get_icon ~icon:M.icon_menu_mlchat ~size:A.LARGE ()
     | Some Other -> A.get_icon ~icon:M.icon_menu_others ~size:A.LARGE ()
     | Some Tools -> A.get_icon ~icon:M.icon_menu_tools ~size:A.LARGE ()
     | Some Bittorrent -> A.get_icon ~icon:M.icon_net_bittorrent ~size:A.LARGE ()
@@ -515,7 +521,7 @@ let save_gui_options list pv =
       (set_simple_option O.gui_ini name v; raise Exit)
     with Exit ->
       (List.iter (fun opt_section ->
-         let section = Options.section_name opt_section in
+         let _section = Options.section_name opt_section in
          try
            List.iter (fun o ->
              if o.option_name = name then
@@ -647,7 +653,7 @@ let rec iter items =
 let config_window gui value_reader f =
   let use_icons = !!O.gtk_look_use_icons in
   let use_avail_bars = !!O.gtk_look_graphical_availability in
-  let language = !!O.gtk_client_lang in
+  let _language = !!O.gtk_client_lang in
   let saturation = !!O.gtk_look_icon_saturation in
   let small_icon = !!O.gtk_look_lists_icon_size in
   let medium_icon = !!O.gtk_look_toolbars_icon_size in
@@ -709,9 +715,8 @@ let config_window gui value_reader f =
       then begin
         let s = U.lang_to_string !!O.gtk_client_lang in
         let ext = String.lowercase s in
-        Printf.printf "gtk_client_lang %s\nlanguage %s\next %s\n"
+        lprintf' "gtk_client_lang %s\nlanguage %s\next %s\n"
           s (U.lang_to_string language) ext;
-        flush stdout;
         Gettext.set_strings_file M.filename ~ext ();
         M.load_messages ();
         GuiStatusBar.update_labels ();
