@@ -147,9 +147,15 @@ let get_port s pos =
 let get_string = get_string16
   
 let get_tag (names_of_tag : (string * field) list) s pos =
-  let t = get_uint8 s pos in
-  let name, pos2 = get_string s (pos+1) in
+  let t2 = get_uint8 s pos in
+  let name, pos2 = 
+    if t2 land 0x80 = 0x80 then
+      String.sub s (pos+1) 1, pos+2
+    else
+      get_string s (pos+1)
+  in
 (*  lprintf "tag name = %s" (String.escaped name);   *)
+  let t = t2 land 0x7f in
   let v, pos = match t with
     | 2 -> let v, pos = get_string s pos2 in
         String v, pos
@@ -161,6 +167,8 @@ let get_tag (names_of_tag : (string * field) list) s pos =
         Uint16 v, pos2 + 2
     | 9 -> let v = get_uint8 s pos2 in
         Uint8 v, pos2 + 1
+    | _ when t >= 0x11 & t <= 0x20 -> let v = String.sub s pos2 (t-0x10) in
+        String v, pos2 + t - 0x10
     | _ -> 
         if !verbose_unknown_messages then
           lprintf "get_tags: unknown tag %d at pos %d\n" t pos;
@@ -171,7 +179,9 @@ let get_tag (names_of_tag : (string * field) list) s pos =
         List.assoc name names_of_tag
       with Not_found ->
 (*          lprintf "Unknown tag \"%s\"\n" (String.escaped name); *)
-          field_of_string name);
+          match field_of_string name with
+          | Field_KNOWN s -> Field_UNKNOWN s
+          | field -> field);
     tag_value = v
   }, pos
   
@@ -203,6 +213,7 @@ let file_common_tags = [
     "\005", Field_Lastseencomplete;
     "\021", Field_Availability;
     "\048", Field_Completesources;
+    "\058", Field_Size_Hi;
     "\208", Field_Artist;
     "\209", Field_Album;
     "\210", Field_Title;
@@ -215,3 +226,77 @@ let file_common_tags = [
     "Album", Field_Album;
     "Title", Field_Title;
   ]
+
+let client_common_tags =
+      [
+        "\001", "name";
+        "\015", "port";
+        "\017", "version";
+        "\031", "udpport";
+        "\032", "compression";
+        "\033", "udpport";
+        "\034", "udpver";
+        "\035", "sourceexchange";
+        "\036", "comments";
+        "\037", "extendedrequest";
+        "\038", "compatibleclient";
+        "\039", "features";
+        "\059", "extrainfo";
+        "\060", "downloadtime";
+        "\061", "incompleteparts";
+        "\062", "l2hac";
+        "\063", "realparts";
+        "\065", "mod_unknown41";
+        "\066", "mod_unknown42";
+        "\067", "mod_unknown43";
+        "\078", "neo_features";
+        "\084", "mod_featureset";
+        "\085", "mod_version";
+        "\086", "mod_protocol";
+        "\090", "mod_bowlfish";
+        "\092", "mod_secure_community";
+        "\093", "mod_unknown0x5d";
+        "\096", "mod_unknown0x60";
+        "\100", "mod_unknown0x64";
+        "\102", "mod_fusion";
+        "\103", "mod_fusion_version";
+
+(* http://forums.shareaza.com/showthread.php?threadid=37323&perpage=15&pagenumber=2 *)
+        "\105", "edonkeyclc serverip?";
+        "\106", "edonkeyclc serverport?";
+
+        "\108", "mod_unknown0x6c";
+        "\117", "mod_unknown0x75"; (* http://emule-project.net @ NewMule *)
+        "\118", "mod_unknown0x76";
+        "\119", "mod_tarod";
+        "\120", "mod_tarod_version";
+        "\121", "mod_morph";
+        "\128", "mod_morph_version";
+        "\130", "mod_mortillo";
+        "\131", "mod_mortillo_version";
+        "\132", "chanblard_version";
+        "\133", "signature";
+        "\134", "cache";
+        "\135", "mod_lsd";
+        "\136", "mod_lsd_version";
+        "\144", "mod_lovelace_version";
+        "\148", "os_info"; (* reused by aMule to transfer client OS type *)
+        "\153", "mod_plus";
+        "\160", "mod_wombat";
+        "\161", "dev_wombat";
+        "\170", "koizo"; (* http://sourceforge.net/projects/koizo *)
+        "\205", "mod_unknown0xcd";
+        "\224", "isp_bypass";
+        "\225", "nat_tunneling";
+        "\239", "emule_compatoptions";
+        "\240", "nat_security";
+        "\249", "emule_udpports";
+        "\250", "emule_miscoptions1";
+        "\251", "emule_version";
+        "\252", "buddy_ip";
+        "\253", "buddy_udp";
+        "\254", "emule_miscoptions2";
+        "pr",   "edonkeyclc horde";
+        "wombia", "wombat a";
+        "wombib", "wombat b";
+      ]
