@@ -52,6 +52,7 @@ and server = {
     mutable server_connected : int64;
     
     mutable server_host : host;
+    mutable server_country_code : int option;
     mutable server_query_key : GnutellaNetwork.query_key;
     mutable server_searches : local_search Fifo.t;
     
@@ -94,6 +95,7 @@ and client = {
     mutable client_all_files : file list option;
     mutable client_requests : download list;
     mutable client_host : (Ip.t * int) option;
+    mutable client_country_code : int option;
     mutable client_reconnect : bool;
     mutable client_connected_for : file option;
     mutable client_support_head_request : bool;
@@ -124,7 +126,6 @@ and file = {
     mutable file_clients : client list;
     mutable file_uids : Uid.t list; 
     mutable file_searches : local_search list;
-    mutable file_filenames : (string * ips_list) list;
     mutable file_clients_queue : client Queues.Queue.t;
     mutable file_nconnected_clients : int;
     mutable file_ttr : TigerTree.t array option;
@@ -136,7 +137,7 @@ and download = {
     mutable download_chunks : (int64 * int64) list;
     mutable download_uploader : CommonSwarming.uploader option;
     mutable download_ranges : download_request list;
-    mutable download_block : CommonSwarming.block option;
+    mutable download_blocks : CommonSwarming.uploader_block list;
     mutable download_head_requested : bool;
     mutable download_ttr_requested : bool;
   }
