@@ -71,15 +71,21 @@ let rec dirname name =
     | n -> String.sub name 0 n
   with Not_found -> "."
 
-      
 let last_extension file =
   try
     let pos = String.rindex file '.' in
-    let pos2 = try String.rindex file slash with _ -> 0 in 
+    let pos2 = try String.rindex file slash with Not_found -> 0 in 
     if pos < pos2 then raise Not_found;
-    let len = String.length file in
-    String.sub file pos (len -pos)
-  with _ -> ""
+    String2.after file pos
+  with Not_found -> ""
+
+let last_extension2 file =
+  try
+    let pos = String.rindex file '.' in
+    let pos2 = try String.rindex file slash with Not_found -> 0 in 
+    if pos < pos2 then raise Not_found;
+    String2.after file (pos + 1)
+  with Not_found -> ""
 
 let extension file =
   try
@@ -109,15 +115,6 @@ let from_string filename =
 let to_string filename =
   List.fold_left (fun file f -> f file) filename !to_strings
 
-  
-let shorten max s =
-  let len = String.length s in
-  if len > max then 
-    Printf.sprintf "%s...%s" (String.sub s 0 (max - 7))
-    (String.sub s (len-4) 4)
-  else s
-    
-    
 let path_of_filename filename =
   let filename = String.copy filename in
   let len = String.length filename in
@@ -141,7 +138,7 @@ let basename filename =
   in
   iter (path_of_filename filename) filename
 
-let filesystem_compliant name fs =
+let filesystem_compliant name fs namemax =
   (* replace all illegal characters with a valid one.
      assumes all filesystems accept '_'s in filenames *)
   let escape_chars p filename =
@@ -202,20 +199,77 @@ let filesystem_compliant name fs =
       minimal_filter c || c = ':' in
     escape_chars macosx_filter name in
 
-  if Autoconf.windows then 
-    windows_compliant name
-  else if Autoconf.system = "macosx" then 
-    macosx_compliant name
-  else 
-    posix_compliant name
+  let sys_checked_name =
+    if Autoconf.windows then 
+      windows_compliant name
+    else if Autoconf.system = "macos" then 
+      macosx_compliant name
+    else 
+      posix_compliant name in
 
-let temp_directory () =
+  let fs_checked_name =
+    let remove_last_spaces s =
+      let len = String.length s in
+      let rec aux n =
+        if n = 0 then n
+        else
+          let n1 = n - 1 in
+          if s.[n1] = ' ' then aux n1
+          else n in
+      let last_space = aux len in
+      if last_space = len then s
+      else String.sub s 0 last_space
+    in
+(* FAT filesystems do not allow files with space as last char *)
+    match fs with
+      "msdos" -> remove_last_spaces sys_checked_name
+    | _ -> sys_checked_name in
+
+  let length_checked_name =
+    if namemax < 1 || String.length sys_checked_name < namemax then
+      fs_checked_name
+    else
+      let ext = extension fs_checked_name in
+        if String.length ext > namemax then
+          String.sub fs_checked_name 0 namemax
+        else
+	  String.sub fs_checked_name 0 (namemax - (String.length ext)) ^ ext
+  in
+  length_checked_name
+
+let temp_dir_name () =
+  try
+    Sys.getenv "MLDONKEY_TEMP"
+  with Not_found ->
+(* kept for compatibility with Filename.temp_dir_name, this code
+   is never reached because $MLDONKEY_TEMP is filled in commonOptions.ml *)
   match Sys.os_type with
-    "Unix" | "Cygwin" ->
+    | "Unix" | "Cygwin" ->
       (try Sys.getenv "TMPDIR" with Not_found -> "/tmp")
-  | "Win32" ->
+    | _ ->
       (try Sys.getenv "TEMP" with Not_found -> ".")
-  | _ -> "."
+
+(* this code is copied from Ocaml stdlib/filename.ml but
+   extended to respect runtime changes to $MLDONKEY_TEMP,
+   Ocaml uses variable $TMPDIR/$TEMP instead *)
+external open_desc: string -> open_flag list -> int -> int = "caml_sys_open"
+external close_desc: int -> unit = "caml_sys_close"
+
+let prng = Random.State.make_self_init ();;
+
+let temp_file_name prefix suffix =
+  let rnd = (Random.State.bits prng) land 0xFFFFFF in
+  Filename.concat (temp_dir_name ()) (Printf.sprintf "%s%06x%s" prefix rnd suffix)
+
+let temp_file prefix suffix =
+  let rec try_name counter =
+    let name = temp_file_name prefix suffix in
+    try
+      close_desc (open_desc name [Open_wronly; Open_creat; Open_excl] 0o600);
+      name
+    with Sys_error _ as e ->
+      if counter >= 1000 then raise e else try_name (counter + 1)
+  in try_name 0
   
 let _ = (* some assertions on these functions *)
   assert (basename "c:\\Program Files\\Toto history.exe" = "Toto history.exe");
