@@ -31,6 +31,9 @@ let tryopen openf closef filename f =
   closef descr;
   result
 
+let exn_drop f x = try f x with _ -> ()
+let with_remove fn f = tryopen (fun fn -> fn) (fun fn -> exn_drop Sys.remove fn) fn f
+
 let tryopen_read fn f = tryopen open_in close_in fn f
 let tryopen_write fn f = tryopen open_out close_out fn f
 let tryopen_read_bin fn f = tryopen open_in_bin close_in fn f
@@ -49,9 +52,13 @@ let tryopen_read_tar fn f =
 let tryopen_write_tar ?compress fn f = 
   tryopen (Tar.open_out ?compress) Tar.close_out fn f
 let tryopen_read_gzip fn f = 
-  tryopen Gzip.open_in Gzip.close_in fn f
+  tryopen Gzip.open_in_file Gzip.close_in fn f
 let tryopen_write_gzip ?level fn f = 
-  tryopen (Gzip.open_out ?level) Gzip.close_out fn f
+  tryopen (Gzip.open_out_file ?level) Gzip.close_out fn f
+let tryopen_umask temp_umask f =
+  (* Unix.umask is not implemented on MinGW *)
+  let safe_umask umask = try Unix.umask umask with Invalid_argument _ -> 0 in
+  tryopen safe_umask (fun oldumask -> ignore(safe_umask oldumask)) temp_umask f
 
 let list_directory filename =
   let list = ref [] in
@@ -204,32 +211,32 @@ let random () =
   done;
   s
 
-let rec can_write_to_directory dirname =
+let can_write_to_directory dirname =
   let temp_file = Filename.concat dirname "tmp_" ^ random () ^ "_mld.tmp" in
-  let check () =
+  let check () = with_remove temp_file (fun _ ->
     tryopen_openfile temp_file [O_WRONLY; O_CREAT] 0o600 (fun fd ->
       let test_string = "mldonkey accesstest - this file can be deleted\n" in
-      really_write fd test_string 0 (String.length test_string));
-    (try Sys.remove temp_file with _ -> ()) in
+      really_write fd test_string 0 (String.length test_string)))
+  in
   try
     check ()
   with
-    | Unix.Unix_error (Unix.EACCES, _, _) ->
-	lprintf_nl "can not create files in directory %s, check rights..." dirname; 
-	exit 73
-    | Unix.Unix_error (Unix.ENOENT, _, _) ->
-	(try
-	  safe_mkdir dirname; 
-	  check ()
-	with _ ->
-          lprintf_nl "%s does not exist and can not be created,  exiting..." dirname; 
-	  exit 73)
-    | Unix.Unix_error (error, what, code) -> 
-	lprintf_nl "%s for directory %s" (error_message error) what;
-	exit 73
-    | e -> 
-	lprintf_nl "%s for directory %s" (Printexc2.to_string e) dirname; 
-	exit 73
+  | Unix.Unix_error (Unix.EACCES, _, _) ->
+      lprintf_nl "can not create files in directory %s, check rights..." dirname; 
+      exit 73
+  | Unix.Unix_error (Unix.ENOENT, _, _) ->
+      (try
+        safe_mkdir dirname;
+        check ()
+      with _ ->
+        lprintf_nl "%s does not exist and can not be created,  exiting..." dirname; 
+        exit 73)
+  | Unix.Unix_error (error, func, what) -> 
+      lprintf_nl "%s(%s) : %s for directory %s" func what (error_message error) dirname;
+      exit 73
+  | e ->
+      lprintf_nl "%s for directory %s" (Printexc2.to_string e) dirname; 
+      exit 73
 
 (** The resource type to query or set with [getrlimit] or [setrlimit] *)
 type rlimit_resource = RLIMIT_CPU (** CPU time in seconds *)
@@ -269,3 +276,6 @@ let ml_setrlimit resource n =
   try
     setrlimit resource new_rlimit
   with _ -> ()
+
+external fsync : Unix.file_descr -> unit = "ml_fsync"
+
