@@ -96,6 +96,7 @@ val font_option : string option_class
 val int_option : int option_class
 val int64_option : int64 option_class
 val percent_option : int option_class
+val port_option : int option_class
 val bool_option : bool option_class
 val float_option : float option_class
 val path_option : string list option_class
@@ -157,14 +158,16 @@ val from_value : 'a option_class -> option_value -> 'a
 
 val value_to_string : option_value -> string
 val string_to_value : string -> option_value
-val option_to_stringvalue : string option -> option_value
-val stringvalue_to_option : option_value -> string option
+val stringoption_to_value : string option -> option_value
+val value_to_stringoption : option_value -> string option
 val value_to_int : option_value -> int
 val int_to_value : int -> option_value
 val value_to_int64 : option_value -> int64
 val int64_to_value : int64 -> option_value
 val value_to_percent : option_value -> int
 val percent_to_value : int -> option_value
+val value_to_port : option_value -> int
+val port_to_value : int -> option_value
 val bool_of_string : string -> bool
 val value_to_bool : option_value -> bool
 val bool_to_value : bool -> option_value
@@ -190,7 +193,6 @@ val value_to_filename : option_value -> string
 val set_simple_option : options_file -> string -> string -> unit
 val simple_options : string -> options_file -> bool -> option_info list
 val get_simple_option : options_file -> string -> string
-val set_option_hook : options_file -> string -> (unit -> unit) -> unit
 
 val set_string_wrappers : 'a option_class -> 
   ('a -> string) -> (string -> 'a) -> unit
