@@ -21,6 +21,8 @@ type t
 val of_inet_addr : Unix.inet_addr -> t
 val of_string : string -> t
 val of_ints : int * int * int * int -> t
+val get_lo16: t -> int
+val get_hi16: t -> int
 
 val to_inet_addr : t -> Unix.inet_addr
 val to_string : t -> string
@@ -32,7 +34,7 @@ val valid : t -> bool
 val local_ip : t -> bool
 val reachable : t -> bool
 val usable : t -> bool
-val banned : (t -> string option) ref
+val banned : (t * int option -> string option) ref
   
 val resolve_one : t -> string
 val matches : t -> t list -> bool
@@ -64,13 +66,13 @@ val equal : t -> t -> bool
 val value_to_ip : Options.option_value -> t
 val ip_to_value : t -> Options.option_value
   
-val async_ip : string -> (t -> unit) -> (int -> unit) -> unit
+val async_ip : string -> (t -> unit) -> (unit -> unit) -> unit
   
 type addr =
   AddrIp of t | AddrName of string
   
 val ip_of_addr : addr -> t
-val async_ip_of_addr : addr -> (t -> unit) -> (int -> unit) -> unit
+val async_ip_of_addr : addr -> (t -> unit) -> (unit -> unit) -> unit
 val string_of_addr : addr -> string
 val addr_of_ip : t -> addr
 val addr_of_string : string -> addr
